<?php
	
	declare(strict_types=1);
	
	/**
	 * @author: daniel
	 * @date: 28.08.24
	 * @time: 07:06
	 */
	
	namespace wpsg\mod_invoice;
	
	use wpsg\wpsg_calculation;
	use wpsg\wpsg_invoice;
	use \wpsg_order;
	
	use horstoeko\zugferd\{
		ZugferdDocumentBuilder, ZugferdProfiles, codelists\ZugferdInvoiceType, ZugferdDocumentPdfBuilder
	};
	
	abstract class eInvoice {
	
		public static function buildUrl(int $invoice_id, int $profil_id): string {
			
			$oInvoice = wpsg_invoice::getInstance($invoice_id);
			$oOrder = $oInvoice->getOrder();
			
			return html_entity_decode(wp_nonce_url(
				WPSG_URL_WP.'wp-admin/admin.php?page=wpsg-Order&action=ajax&noheader=1&mod=wpsg_mod_rechnungen&do=getRechnung&edit_id='.$oOrder->getId().'&r_id='.$oInvoice->getId().'&profil_id='.$profil_id, 'wpsg-mod_invoice-order_ajax-getRechnung-'.$oInvoice->getId()
			));
			
		}
		
		public static function writeEInvoice(int $order_id, int $invoice_id, int $profil_id = 0): string {
			
			$oInvoice = wpsg_invoice::getInstance($invoice_id);
			
			$shop = \wpsg_ShopController::getShop();
			
			if ($profil_id === 0) $profil_id = intval($shop->get_option('wpsg_mod_rechnungen_einvoice')) - 100;
			
			if ($profil_id >= 0) {
			
				$oOrder = wpsg_order::getInstance($order_id);
				
				$pdf_file = $oInvoice->getFilePath();
				$pdf_path = dirname($pdf_file).DIRECTORY_SEPARATOR;
				
				$document = ZugferdDocumentBuilder::CreateNew($profil_id);
				
				$document
					->setDocumentInformation($oInvoice->getNr(), ZugferdInvoiceType::INVOICE, \DateTime::createFromFormat("Ymd", $oInvoice->getDate('Ymd')), "EUR")
					->addDocumentNote(wpsg_translate(__('Rechnung #1# zu Bestellung #2#', 'wpsg'), $oInvoice->getNr(), $oOrder->getNr()))
					// ->setDocumentSupplyChainEvent(\DateTime::createFromFormat('Ymd', '20180305')) Lieferinformation
					->setDocumentSeller($shop->get_option('wpsg_shopdata_name', default: ''))
					->addDocumentSellerContact(
						$shop->get_option('wpsg_shopdata_owner', default: ''),
						null,
						null,
						null,
						$shop->get_option('wpsg_shopdata_email', default: '')
					)
					->setDocumentSellerContact(
						$shop->get_option('wpsg_shopdata_owner', default: ''),
						null,
						null,
						null,
						$shop->get_option('wpsg_shopdata_email', default: '')
					)
					//->addDocumentSellerGlobalId("4000001123452", "0088")
					->addDocumentSellerTaxRegistration("FC", $shop->get_option('wpsg_shopdata_taxnr', default: ''))
					->addDocumentSellerTaxRegistration("VA", $shop->get_option('wpsg_shopdata_ustidnr', default: ''))
					->setDocumentSellerAddress(
						$shop->get_option('wpsg_shopdata_street', default: ''),
						"",
						"",
						$shop->get_option('wpsg_shopdata_zip', default: ''),
						$shop->get_option('wpsg_shopdata_city', default: ''),
						\wpsg_country::getInstance(intval($shop->get_option('wpsg_shopdata_country')))->getShorttext()
					)
					->setDocumentBuyer($oOrder->getInvoiceSalutationTitle(), strval($oOrder->getId()))
					->setDocumentBuyerAddress(
						$oOrder->getInvoiceStreet(),
						"",
						"",
						$oOrder->getInvoiceZip(),
						$oOrder->getInvoiceCity(),
						$oOrder->getInvoiceCountryKuerzel()
					);
				
				// Steuer Anteile
				foreach ($oInvoice->getTaxInfo() as $tax_info) {
				
					$document->addDocumentTax('S', 'VAT', $tax_info['netto'], $tax_info['tax_value'], $tax_info['tax_rate']);
					
				}
				
				$document->setDocumentSummation(
					$oInvoice->getSumBrutto(),
					$oInvoice->getSumBrutto(),
					$oInvoice->getSumNetto(),
					0.0,
					0.0,
					$oInvoice->getSumNetto(),
					$oInvoice->getSumBrutto() - $oInvoice->getSumNetto()
				);
				
				// $document->addDocumentPaymentTerm("Zahlbar innerhalb 30 Tagen netto bis 04.04.2018, 3% Skonto innerhalb 10 Tagen bis 15.03.2018")
				
				foreach ($oInvoice->getItems() as $oInvoiceItem) {
				
					$lineID = ($lineID??0) + 1;
					
					$document->addNewPosition(strval($lineID));
					$document->setDocumentPositionProductDetails(
						$oInvoiceItem->getLabel(),
						"",
						strval($oInvoiceItem->getId()),
						null,
						"0160", // GS1 Global Trade Item Number (GTIN, EAN)
						$oInvoiceItem->getGTIN()
					);
					$document->setDocumentPositionGrossPrice($oInvoiceItem->getBrutto());
					$document->setDocumentPositionNetPrice($oInvoiceItem->getNetto());
					$document->setDocumentPositionQuantity($oInvoiceItem->getAmount(), "H87");
					$document->addDocumentPositionTax('S', 'VAT', $oInvoiceItem->getTaxRate());
					$document->setDocumentPositionLineSummation($oInvoiceItem->getBruttoSum());
					
				}
				
				$document->writeFile($pdf_path."/e".$oInvoice->getId()."_".$profil_id.".xml");
				
				$pdfBuilder = new ZugferdDocumentPdfBuilder($document, $pdf_file);
				
				$pdf_path .= 'e'.$oInvoice->getId().'_'.$profil_id.'.pdf';
				
  				$pdfBuilder->generateDocument()->saveDocument($pdf_path);
				 
				return $pdf_path;
				
				//\wpsg_header::PDFPlugin($pdf_path);
				//die("OK".$pdf_path);
				
			}
			
			return $oInvoice->getFilePath();
			
		}
	
	}